varying highp vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform lowp float hueValue;

const highp vec4 kRGB_to_y_prime = vec4 (0.299, 0.587, 0.114, 0.0);
const highp vec4 kRGB_to_i = vec4 (0.595716, -0.274453, -0.321263, 0.0);
const highp vec4 kRGBT_to_q = vec4 (0.211456, -0.522591, 0.31135, 0.0);

const highp vec4 kYIQ_to_r = vec4 (1.0, 0.9563, 0.6210, 0.0);
const highp vec4 kYIQ_to_g = vec4 (1.0, -0.2721, -0.6474, 0.0);
const highp vec4 kYIQ_to_b = vec4 (1.0, -1.1070, 1.7046, 0.0);

void main() {
    lowp vec4 textureColor = texture2D(inputImageTexture, textureCoordinate);

    highp float y_prime = dot(textureColor, kRGB_to_y_prime);
    highp float i = dot(textureColor, kRGB_to_i);
    highp float q = dot(textureColor, kRGBT_to_q);

    highp float hue = atan(q, i);
    highp float chroma = sqrt(i * i + q * q);
    hue -= hueValue;

    q = chroma * sin(hue);
    i = chroma * cos(hue);

    highp vec4 y_i_q = vec4(y_prime, i, q, 0.0);
    textureColor.x = dot(y_i_q, kYIQ_to_r);
    textureColor.y = dot(y_i_q, kYIQ_to_g);
    textureColor.z = dot(y_i_q, kYIQ_to_b);

    gl_FragColor = textureColor;
}